@@ -1,7 +1,7 @@
 /*
  Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>
  Copyright (C) 2008-2010 Arno Renevier <arno@renevier.net>
- Copyright (C) 2010 Paweł Forysiuk <tuxator@o2.pl>
+ Copyright (C) 2010-2011 Paweł Forysiuk <tuxator@o2.pl>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -100,11 +100,7 @@ addons_install_response (GtkWidget*  infobar,
 {
     if (response_id == GTK_RESPONSE_ACCEPT)
     {
-        MidoriBrowser* browser;
-        const gchar* uri;
-
-        browser = midori_browser_get_for_widget (GTK_WIDGET (infobar));
-        uri = midori_view_get_display_uri (view);
+        const gchar* uri = midori_view_get_display_uri (view);
         if (uri && *uri)
         {
             gchar** split_uri;
@@ -152,15 +148,10 @@ addons_install_response (GtkWidget*  infobar,
             }
             else if (!g_strcmp0 (hostname, "userstyles.org"))
             {
-                gchar* subpage;
+                gchar* subpage = split_uri[4];
 
                 folder = "styles";
-                if (g_str_has_suffix (uri, "/"))
-                    subpage = split_uri[6];
-                else
-                    subpage = split_uri[5];
-
-                if (!subpage)
+                if ((subpage && *subpage) && g_ascii_isdigit (subpage[0]))
                 {
                     gchar* style_id;
                     const gchar* js_script;
@@ -278,14 +269,10 @@ addons_notify_load_status_cb (MidoriView*      view,
            else if (g_str_has_prefix (uri, "http://userstyles.org/styles/"))
            {
                gchar** split_uri = g_strsplit (uri, "/", -1);
-               gchar* subpage;
+               gchar* subpage = split_uri[4];
 
-               if (g_str_has_suffix (uri, "/"))
-                   subpage = split_uri[6];
-               else
-                   subpage = split_uri[5];
                /* userstyles.org style main page with style description */
-               if (!subpage)
+               if ((subpage && *subpage) && g_ascii_isdigit (subpage[0]))
                    addons_uri_install (view, ADDONS_USER_STYLES);
 
                g_strfreev (split_uri);
@@ -674,8 +661,6 @@ addons_get_toolbar (MidoriViewable* viewable)
         g_signal_connect (toolitem, "clicked",
             G_CALLBACK (midori_addons_button_delete_clicked_cb), viewable);
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
-        gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem),
-                                    _("Open target folder for selected addon"));
         gtk_widget_set_tooltip_text (GTK_WIDGET (toolitem), _("Remove selected addon"));
         gtk_widget_show (GTK_WIDGET (toolitem));
         ADDONS (viewable)->toolbar = toolbar;
