@@ -16,6 +16,7 @@
 #endif
 
 #include "sokoke.h"
+#include "midori-stock.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -196,34 +197,6 @@ midori_preferences_notify_proxy_type_cb (MidoriWebSettings* settings,
 }
 #endif
 
-static void
-midori_preferences_delete_cookies_toggled_cb (GtkToggleButton*   button,
-                                              MidoriWebSettings* settings)
-{
-    gboolean toggled = gtk_toggle_button_get_active (button);
-    g_object_set (settings, "accept-cookies",
-        toggled ? MIDORI_ACCEPT_COOKIES_SESSION : MIDORI_ACCEPT_COOKIES_ALL, NULL);
-}
-
-static void
-midori_preferences_delete_cookies_changed_cb (GtkComboBox*       combo,
-                                              MidoriWebSettings* settings)
-{
-    gint active = gtk_combo_box_get_active (combo);
-    gint max_age;
-    switch (active)
-    {
-    case 0: max_age =   0; break;
-    case 1: max_age =   1; break;
-    case 2: max_age =   7; break;
-    case 3: max_age =  30; break;
-    case 4: max_age = 365; break;
-    default:
-        max_age = 30;
-    }
-    g_object_set (settings, "maximum-cookie-age", max_age, NULL);
-}
-
 #if HAVE_OSX
 static void
 midori_preferences_toolbutton_clicked_cb (GtkWidget* toolbutton,
@@ -326,8 +299,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     #define SPANNED_ADD(__widget) \
      katze_preferences_add_widget (_preferences, __widget, "spanned")
     /* Page "General" */
-    PAGE_NEW (GTK_STOCK_HOME, _("General"));
-    FRAME_NEW (_("Startup"));
+    PAGE_NEW (GTK_STOCK_HOME, _("Startup"));
+    FRAME_NEW (NULL);
     label = katze_property_label (settings, "load-on-startup");
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "load-on-startup", NULL);
@@ -355,21 +328,10 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         SPANNED_ADD (button);
         #endif
     }
-    button = katze_property_proxy (settings, "show-crash-dialog", NULL);
-    INDENTED_ADD (button);
-    button = katze_property_proxy (settings, "speed-dial-in-new-tabs", NULL);
-    SPANNED_ADD (button);
-    FRAME_NEW (_("Transfers"));
-    #if !HAVE_HILDON
-    label = katze_property_label (settings, "download-folder");
-    INDENTED_ADD (label);
-    button = katze_property_proxy (settings, "download-folder", "folder");
-    SPANNED_ADD (button);
-    #endif
 
     /* Page "Appearance" */
     PAGE_NEW (GTK_STOCK_SELECT_FONT, _("Appearance"));
-    FRAME_NEW (_("Font settings"));
+    FRAME_NEW (NULL);
     #if !HAVE_HILDON
     label = gtk_label_new (_("Default Font Family"));
     INDENTED_ADD (label);
@@ -400,7 +362,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
 
     /* Page "Behavior" */
     PAGE_NEW (GTK_STOCK_SELECT_COLOR, _("Behavior"));
-    FRAME_NEW (_("Features"));
+    FRAME_NEW (NULL);
     #if !HAVE_HILDON
     button = katze_property_proxy (settings, "auto-load-images", NULL);
     INDENTED_ADD (button);
@@ -439,16 +401,25 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     INDENTED_ADD (button);
     entry = katze_property_proxy (settings, "preferred-languages", "languages");
     SPANNED_ADD (entry);
-
-    /* Page "Interface" */
-    PAGE_NEW (GTK_STOCK_CONVERT, _("Interface"));
-    FRAME_NEW (_("Navigationbar"));
     #if !HAVE_HILDON
-    INDENTED_ADD (katze_property_label (settings, "toolbar-style"));
-    button = katze_property_proxy (settings, "toolbar-style", NULL);
+    label = katze_property_label (settings, "download-folder");
+    INDENTED_ADD (label);
+    button = katze_property_proxy (settings, "download-folder", "folder");
     SPANNED_ADD (button);
     #endif
-    FRAME_NEW (_("Browsing"));
+
+    /* Page "Interface" */
+    PAGE_NEW (GTK_STOCK_CONVERT, _("Browsing"));
+    #if !HAVE_HILDON
+    if (!g_getenv ("DESKTOP_SESSION"))
+    {
+        FRAME_NEW (NULL);
+        INDENTED_ADD (katze_property_label (settings, "toolbar-style"));
+        button = katze_property_proxy (settings, "toolbar-style", NULL);
+        SPANNED_ADD (button);
+    }
+    #endif
+    FRAME_NEW (NULL);
     label = katze_property_label (settings, "open-new-pages-in");
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "open-new-pages-in", NULL);
@@ -464,18 +435,14 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     button = katze_property_proxy (settings, "open-tabs-in-the-background", NULL);
     SPANNED_ADD (button);
 
-    #if !HAVE_HILDON
     /* Page "Applications" */
+    #if !HAVE_HILDON
     PAGE_NEW (GTK_STOCK_CONVERT, _("Applications"));
-    FRAME_NEW (_("External applications"));
+    FRAME_NEW (NULL);
     label = katze_property_label (settings, "text-editor");
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "text-editor", "application-text/plain");
     SPANNED_ADD (entry);
-    label = katze_property_label (settings, "download-manager");
-    INDENTED_ADD (label);
-    entry = katze_property_proxy (settings, "download-manager", "application-FileTransfer");
-    SPANNED_ADD (entry);
     label = katze_property_label (settings, "news-aggregator");
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "news-aggregator", "application-News");
@@ -484,7 +451,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
 
     /* Page "Network" */
     PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
-    FRAME_NEW (_("Network"));
+    FRAME_NEW (NULL);
     #if !HAVE_HILDON
     label = katze_property_label (settings, "proxy-type");
     INDENTED_ADD (label);
@@ -498,67 +465,16 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry);
     midori_preferences_notify_proxy_type_cb (settings, NULL, entry);
     #endif
+    #if WEBKIT_CHECK_VERSION (1, 3, 11)
+    label = katze_property_label (settings, "maximum-cache-size");
+    INDENTED_ADD (label);
+    button = katze_property_proxy (settings, "maximum-cache-size", NULL);
+    SPANNED_ADD (button);
+    label = gtk_label_new (_("MB"));
+    SPANNED_ADD (label);
+    #endif
     label = katze_property_label (settings, "identify-as");
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "identify-as", "custom-user-agent");
     SPANNED_ADD (button);
-
-    /* Page "Privacy" */
-    PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
-    FRAME_NEW (_("Web Cookies"));
-    button = gtk_check_button_new_with_mnemonic (_("Delete cookies when quitting Midori"));
-    INDENTED_ADD (button);
-    if (katze_object_get_enum (settings, "accept-cookies") == MIDORI_ACCEPT_COOKIES_SESSION)
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-    g_signal_connect (button, "toggled",
-        G_CALLBACK (midori_preferences_delete_cookies_toggled_cb), settings);
-    button = gtk_combo_box_new_text ();
-    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 hour"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 day"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 week"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 month"));
-    gtk_combo_box_append_text (GTK_COMBO_BOX (button), _("Delete old cookies after 1 year"));
-    {
-        gint max_age = katze_object_get_int (settings, "maximum-cookie-age");
-        guint active;
-        switch (max_age)
-        {
-        case   0: active = 0; break;
-        case   1: active = 1; break;
-        case   7: active = 2; break;
-        case  30: active = 3; break;
-        case 365: active = 4; break;
-        default:
-            active = 3;
-        }
-        gtk_combo_box_set_active (GTK_COMBO_BOX (button), active);
-    }
-    g_signal_connect (button, "changed",
-        G_CALLBACK (midori_preferences_delete_cookies_changed_cb), settings);
-    SPANNED_ADD (button);
-    {
-        gchar* markup = g_strdup_printf ("<span size=\"smaller\">%s</span>",
-            _("Cookies store login data, saved games, "
-              "or user profiles for advertisement purposes."));
-        label = gtk_label_new (NULL);
-        gtk_label_set_markup (GTK_LABEL (label), markup);
-        g_free (markup);
-    }
-    FILLED_ADD (label);
-    #if WEBKIT_CHECK_VERSION (1, 1, 13)
-    INDENTED_ADD (katze_property_proxy (settings, "enable-offline-web-application-cache", NULL));
-    #endif
-    #if WEBKIT_CHECK_VERSION (1, 1, 8)
-    SPANNED_ADD (katze_property_proxy (settings, "enable-html5-local-storage", NULL));
-    #if !WEBKIT_CHECK_VERSION (1, 1, 14)
-    INDENTED_ADD (katze_property_proxy (settings, "enable-html5-database", NULL));
-    #endif
-    #endif
-    FRAME_NEW (_("History"));
-    button = katze_property_label (settings, "maximum-history-age");
-    INDENTED_ADD (button);
-    button = katze_property_proxy (settings, "maximum-history-age", NULL);
-    SPANNED_ADD (button);
-    label = gtk_label_new (_("days"));
-    SPANNED_ADD (label);
 }
